@@ -29,10 +29,11 @@
 #include <CalendarSupport/KCalPrefs>
 #include <CalendarSupport/Utils>
 
-#include <Akonadi/Calendar/ETMCalendar>
-#include <AkonadiCore/EntityTreeModel>
-#include <CollectionComboBox>
-#include <Item>
+#include <Akonadi/CalendarUtils>
+#include <Akonadi/CollectionComboBox>
+#include <Akonadi/ETMCalendar>
+#include <Akonadi/EntityTreeModel>
+#include <Akonadi/Item>
 
 #include <KCalUtils/Stringify>
 #include <KCalendarCore/ICalFormat>
@@ -40,15 +41,21 @@
 
 #include <KMessageBox>
 #include <KSharedConfig>
+#include <kwidgetsaddons_version.h>
 
+#include <KWindowConfig>
 #include <QCloseEvent>
 #include <QDir>
 #include <QIcon>
 #include <QStandardPaths>
 #include <QTimeZone>
+#include <QWindow>
 
 using namespace IncidenceEditorNG;
-
+namespace
+{
+static const char myIncidenceDialogConfigGroupName[] = "IncidenceDialog";
+}
 namespace IncidenceEditorNG
 {
 enum Tabs { GeneralTab = 0, AttendeesTab, ResourcesTab, AlarmsTab, RecurrenceTab, AttachmentsTab };
@@ -93,15 +100,15 @@ public:
     void setCalendarCollection(const Akonadi::Collection &collection);
 
     /// ItemEditorUi methods
-    bool containsPayloadIdentifiers(const QSet<QByteArray> &partIdentifiers) const override;
+    Q_REQUIRED_RESULT bool containsPayloadIdentifiers(const QSet<QByteArray> &partIdentifiers) const override;
     void handleItemSaveFinish(EditorItemManager::SaveAction);
     void handleItemSaveFail(EditorItemManager::SaveAction, const QString &errorMessage);
-    bool hasSupportedPayload(const Akonadi::Item &item) const override;
-    bool isDirty() const override;
-    bool isValid() const override;
+    Q_REQUIRED_RESULT bool hasSupportedPayload(const Akonadi::Item &item) const override;
+    Q_REQUIRED_RESULT bool isDirty() const override;
+    Q_REQUIRED_RESULT bool isValid() const override;
     void load(const Akonadi::Item &item) override;
     Akonadi::Item save(const Akonadi::Item &item) override;
-    Akonadi::Collection selectedCollection() const override;
+    Q_REQUIRED_RESULT Akonadi::Collection selectedCollection() const override;
 
     void reject(RejectReason reason, const QString &errorMessage = QString()) override;
 };
@@ -166,19 +173,33 @@ IncidenceDialogPrivate::IncidenceDialogPrivate(Akonadi::IncidenceChanger *change
     const Akonadi::Collection col(colId);
     setCalendarCollection(col);
 
-    q->connect(mEditor, SIGNAL(showMessage(QString, KMessageWidget::MessageType)), SLOT(showMessage(QString, KMessageWidget::MessageType)));
-    q->connect(mEditor, SIGNAL(dirtyStatusChanged(bool)), SLOT(updateButtonStatus(bool)));
-    q->connect(mItemManager,
-               SIGNAL(itemSaveFinished(IncidenceEditorNG::EditorItemManager::SaveAction)),
-               SLOT(handleItemSaveFinish(IncidenceEditorNG::EditorItemManager::SaveAction)));
-    q->connect(mItemManager,
-               SIGNAL(itemSaveFailed(IncidenceEditorNG::EditorItemManager::SaveAction, QString)),
-               SLOT(handleItemSaveFail(IncidenceEditorNG::EditorItemManager::SaveAction, QString)));
-    q->connect(ieAlarm, SIGNAL(alarmCountChanged(int)), SLOT(handleAlarmCountChange(int)));
-    q->connect(mIeRecurrence, SIGNAL(recurrenceChanged(IncidenceEditorNG::RecurrenceType)), SLOT(handleRecurrenceChange(IncidenceEditorNG::RecurrenceType)));
-    q->connect(ieAttachments, SIGNAL(attachmentCountChanged(int)), SLOT(updateAttachmentCount(int)));
-    q->connect(mIeAttendee, SIGNAL(attendeeCountChanged(int)), SLOT(updateAttendeeCount(int)));
-    q->connect(mIeResource, SIGNAL(resourceCountChanged(int)), SLOT(updateResourceCount(int)));
+    q->connect(mEditor, &CombinedIncidenceEditor::showMessage, q, [this](const QString &reason, KMessageWidget::MessageType msgType) {
+        showMessage(reason, msgType);
+    });
+    q->connect(mEditor, &IncidenceEditor::dirtyStatusChanged, q, [this](bool isDirty) {
+        updateButtonStatus(isDirty);
+    });
+    q->connect(mItemManager, &EditorItemManager::itemSaveFinished, q, [this](EditorItemManager::SaveAction action) {
+        handleItemSaveFinish(action);
+    });
+    q->connect(mItemManager, &EditorItemManager::itemSaveFailed, q, [this](EditorItemManager::SaveAction action, const QString &message) {
+        handleItemSaveFail(action, message);
+    });
+    q->connect(ieAlarm, &IncidenceAlarm::alarmCountChanged, q, [this](int newCount) {
+        handleAlarmCountChange(newCount);
+    });
+    q->connect(mIeRecurrence, &IncidenceRecurrence::recurrenceChanged, q, [this](IncidenceEditorNG::RecurrenceType type) {
+        handleRecurrenceChange(type);
+    });
+    q->connect(ieAttachments, &IncidenceAttachment::attachmentCountChanged, q, [this](int newCount) {
+        updateAttachmentCount(newCount);
+    });
+    q->connect(mIeAttendee, &IncidenceAttendee::attendeeCountChanged, q, [this](int count) {
+        updateAttendeeCount(count);
+    });
+    q->connect(mIeResource, &IncidenceResource::resourceCountChanged, q, [this](int count) {
+        updateResourceCount(count);
+    });
 }
 
 IncidenceDialogPrivate::~IncidenceDialogPrivate()
@@ -314,9 +335,15 @@ void IncidenceDialogPrivate::manageTemplates()
     QPointer<IncidenceEditorNG::TemplateManagementDialog> dialog(
         new IncidenceEditorNG::TemplateManagementDialog(q, templates, KCalUtils::Stringify::incidenceType(mEditor->type())));
 
-    q->connect(dialog, SIGNAL(loadTemplate(QString)), SLOT(loadTemplate(QString)));
-    q->connect(dialog, SIGNAL(templatesChanged(QStringList)), SLOT(storeTemplatesInConfig(QStringList)));
-    q->connect(dialog, SIGNAL(saveTemplate(QString)), SLOT(saveTemplate(QString)));
+    q->connect(dialog, &TemplateManagementDialog::loadTemplate, q, [this](const QString &templateName) {
+        loadTemplate(templateName);
+    });
+    q->connect(dialog, &TemplateManagementDialog::templatesChanged, q, [this](const QStringList &templates) {
+        storeTemplatesInConfig(templates);
+    });
+    q->connect(dialog, &TemplateManagementDialog::saveTemplate, q, [this](const QString &templateName) {
+        saveTemplate(templateName);
+    });
     dialog->exec();
     delete dialog;
 }
@@ -429,7 +456,20 @@ void IncidenceDialogPrivate::handleItemSaveFail(EditorItemManager::SaveAction, c
                                       "Unable to store the incidence in the calendar. Try again?\n\n "
                                       "Reason: %1",
                                       errorMessage);
-        retry = (KMessageBox::warningYesNo(q, message) == KMessageBox::Yes);
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        const int answer = KMessageBox::warningTwoActions(q,
+#else
+        const int answer = KMessageBox::warningYesNo(q,
+#endif
+                                                          message,
+                                                          QString(),
+                                                          KGuiItem(i18nc("@action:button", "Retry"), QStringLiteral("dialog-ok")),
+                                                          KStandardGuiItem::cancel());
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        retry = (answer == KMessageBox::ButtonCode::PrimaryAction);
+#else
+        retry = (answer == KMessageBox::Yes);
+#endif
     }
 
     if (retry) {
@@ -455,13 +495,21 @@ void IncidenceDialogPrivate::handleItemSaveFinish(EditorItemManager::SaveAction
                                        "<para>Would you like to set your default events calendar to "
                                        "<resource>%1</resource>?</para>",
                                        collectionName);
-        if (KMessageBox::questionYesNo(q,
-                                       message,
-                                       i18nc("@title:window", "Set Default Calendar?"),
-                                       KStandardGuiItem::yes(), // Make collectionName My Default Calendar
-                                       KStandardGuiItem::no(), // Do Not Set a Default Calendar at this Time"
-                                       QStringLiteral("setDefaultCalendarCollection"))
-            == KMessageBox::Yes) {
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        const int answer = KMessageBox::questionTwoActions(q,
+#else
+        const int answer = KMessageBox::questionYesNo(q,
+#endif
+                                                           message,
+                                                           i18nc("@title:window", "Set Default Calendar?"),
+                                                           KGuiItem(i18nc("@action:button", "Set As Default"), QStringLiteral("dialog-ok")),
+                                                           KGuiItem(i18nc("@action:button", "Do Not Set"), QStringLiteral("dialog-cancel")),
+                                                           QStringLiteral("setDefaultCalendarCollection"));
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        if (answer == KMessageBox::ButtonCode::PrimaryAction) {
+#else
+        if (answer == KMessageBox::Yes) {
+#endif
             CalendarSupport::KCalPrefs::instance()->setDefaultCalendarId(mItem.storageCollectionId());
         }
     }
@@ -492,7 +540,7 @@ void IncidenceDialogPrivate::handleItemSaveFinish(EditorItemManager::SaveAction
 
 bool IncidenceDialogPrivate::hasSupportedPayload(const Akonadi::Item &item) const
 {
-    return CalendarSupport::incidence(item);
+    return !Akonadi::CalendarUtils::incidence(item).isNull();
 }
 
 bool IncidenceDialogPrivate::isDirty() const
@@ -535,10 +583,10 @@ void IncidenceDialogPrivate::load(const Akonadi::Item &item)
         mUi->mTabWidget->removeTab(ResourcesTab);
     }
 
-    mEditor->load(CalendarSupport::incidence(item));
+    mEditor->load(Akonadi::CalendarUtils::incidence(item));
     mEditor->load(item);
 
-    const KCalendarCore::Incidence::Ptr incidence = CalendarSupport::incidence(item);
+    const KCalendarCore::Incidence::Ptr incidence = Akonadi::CalendarUtils::incidence(item);
     const QStringList allEmails = IncidenceEditorNG::EditorConfig::instance()->allEmails();
     const KCalendarCore::Attendee me = incidence->attendeeByMails(allEmails);
 
@@ -663,31 +711,30 @@ IncidenceDialog::IncidenceDialog(Akonadi::IncidenceChanger *changer, QWidget *pa
     connect(d->mUi->mAcceptInvitationButton, &QAbstractButton::clicked, d->mUi->mInvitationBar, &QWidget::hide);
     connect(d->mUi->mDeclineInvitationButton, &QAbstractButton::clicked, d->mIeAttendee, &IncidenceAttendee::declineForMe);
     connect(d->mUi->mDeclineInvitationButton, &QAbstractButton::clicked, d->mUi->mInvitationBar, &QWidget::hide);
-    connect(this, SIGNAL(invalidCollection()), this, SLOT(slotInvalidCollection()));
+    connect(this, &IncidenceDialog::invalidCollection, this, [d]() {
+        d->slotInvalidCollection();
+    });
     readConfig();
 }
 
 IncidenceDialog::~IncidenceDialog()
 {
     writeConfig();
-    delete d_ptr;
 }
 
 void IncidenceDialog::writeConfig()
 {
-    KConfigGroup group(KSharedConfig::openConfig(), "IncidenceDialog");
-    group.writeEntry("Size", size());
+    KConfigGroup group(KSharedConfig::openStateConfig(), myIncidenceDialogConfigGroupName);
+    KWindowConfig::saveWindowSize(windowHandle(), group);
 }
 
 void IncidenceDialog::readConfig()
 {
-    KConfigGroup group(KSharedConfig::openConfig(), "IncidenceDialog");
-    const QSize size = group.readEntry("Size", QSize());
-    if (size.isValid()) {
-        resize(size);
-    } else {
-        resize(QSize(500, 500).expandedTo(minimumSizeHint()));
-    }
+    create(); // ensure a window is created
+    windowHandle()->resize(QSize(500, 500));
+    KConfigGroup group(KSharedConfig::openStateConfig(), myIncidenceDialogConfigGroupName);
+    KWindowConfig::restoreWindowSize(windowHandle(), group);
+    resize(windowHandle()->size()); // workaround for QTBUG-40584
 }
 
 void IncidenceDialog::load(const Akonadi::Item &item, const QDate &activeDate)
@@ -746,8 +793,20 @@ void IncidenceDialog::slotButtonClicked(QAbstractButton *button)
         d->mItemManager->save();
     } else if (d->mUi->buttonBox->button(QDialogButtonBox::Cancel) == button) {
         if (d->isDirty()
-            && KMessageBox::questionYesNo(this, i18nc("@info", "Do you really want to cancel?"), i18nc("@title:window", "KOrganizer Confirmation"))
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+            && KMessageBox::questionTwoActions(this,
+#else
+            && KMessageBox::questionYesNo(this,
+#endif
+                                               i18nc("@info", "Do you really want to cancel?"),
+                                               i18nc("@title:window", "KOrganizer Confirmation"),
+                                               KGuiItem(i18nc("@action:button", "Cancel Editing"), QStringLiteral("dialog-ok")),
+                                               KGuiItem(i18nc("@action:button", "Do Not Cancel"), QStringLiteral("dialog-cancel")))
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+                == KMessageBox::ButtonCode::PrimaryAction) {
+#else
                 == KMessageBox::Yes) {
+#endif
             QDialog::reject(); // Discard current changes
         } else if (!d->isDirty()) {
             QDialog::reject(); // No pending changes, just close the dialog.
@@ -763,8 +822,20 @@ void IncidenceDialog::reject()
 {
     Q_D(IncidenceDialog);
     if (d->isDirty()
-        && KMessageBox::questionYesNo(this, i18nc("@info", "Do you really want to cancel?"), i18nc("@title:window", "KOrganizer Confirmation"))
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        && KMessageBox::questionTwoActions(this,
+#else
+        && KMessageBox::questionYesNo(this,
+#endif
+                                           i18nc("@info", "Do you really want to cancel?"),
+                                           i18nc("@title:window", "KOrganizer Confirmation"),
+                                           KGuiItem(i18nc("@action:button", "Cancel Editing"), QStringLiteral("dialog-ok")),
+                                           KGuiItem(i18nc("@action:button", "Do Not Cancel"), QStringLiteral("dialog-cancel")))
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+            == KMessageBox::ButtonCode::PrimaryAction) {
+#else
             == KMessageBox::Yes) {
+#endif
         QDialog::reject(); // Discard current changes
     } else if (!d->isDirty()) {
         QDialog::reject(); // No pending changes, just close the dialog.
@@ -775,8 +846,20 @@ void IncidenceDialog::closeEvent(QCloseEvent *event)
 {
     Q_D(IncidenceDialog);
     if (d->isDirty()
-        && KMessageBox::questionYesNo(this, i18nc("@info", "Do you really want to cancel?"), i18nc("@title:window", "KOrganizer Confirmation"))
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        && KMessageBox::questionTwoActions(this,
+#else
+        && KMessageBox::questionYesNo(this,
+#endif
+                                           i18nc("@info", "Do you really want to cancel?"),
+                                           i18nc("@title:window", "KOrganizer Confirmation"),
+                                           KGuiItem(i18nc("@action:button", "Cancel Editing"), QStringLiteral("dialog-ok")),
+                                           KGuiItem(i18nc("@action:button", "Do Not Cancel"), QStringLiteral("dialog-cancel")))
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+            == KMessageBox::ButtonCode::PrimaryAction) {
+#else
             == KMessageBox::Yes) {
+#endif
         QDialog::reject(); // Discard current changes
         QDialog::closeEvent(event);
     } else if (!d->isDirty()) {
